﻿--[[
WarlockReminder Continued v1.60 
09 July 2011
Author: CKlausi/Biny of Anetheron EU
Original Author of WarlockReminder: Vux of Dragonblight-US
Description: Helpful reminders for Warlocks
File: WarlockReminder.lua
]]--

WarlockReminder = LibStub("AceAddon-3.0"):NewAddon("WarlockReminder", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("WarlockReminder")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local media = LibStub("LibSharedMedia-3.0")
local textures = media:List("statusbar")
local fonts = media:List("font")

-- Default Values
local defaults = {
    profile = {
		--Reminder
		showArmor = true,
		showSL = true,
		showFlask = true,
		showPet = true,
		showFood = true,
		-- Pets
		petAffliction = "felhunter",
		petDemonology = "felguard",
		petDestruction = "imp",
		-- DS
		playDrainSoul = true,
		showDrainSoulMessage = true,
		drainSoulPercent = 25 / 100,
		drainSoulHealth = 100000,
		showDrainSoulFrame = true,
		drainSoulDuration = 5,
		-- Output
		outputFrame = "wlrframe",
		-- Anchor
		wlrAnchorTop = UIParent:GetHeight()/2, 
		wlrAnchorLeft = UIParent:GetWidth()/2,
		wlrAnchorAlpha = 1,
		wlrAnchorSize = 1,
		-- Decimation
		playDecimationSound = true,
		showDecimationMessage = true,
		showDecimationFrame = true,
		-- MoltenCore
		playMoltenCoreSound = false,
		showMoltenCoreMessage = false,
		showMoltenCoreFrame = false,
		-- Shadow Trance
		playShadowTranceSound = false,
		showShadowTranceMessage = false,
		showShadowTranceFrame = false,
		-- Empowered Imp
		playEmpoweredImpSound = false,
		showEmpoweredImpMessage = false,
		showEmpoweredImpFrame = false,
		-- Dark Intent
		showDarkIntentFrame = true,
		diFrameTop = (UIParent:GetHeight()/2) +40,
		diFrameLeft = (UIParent:GetWidth()/2),
		diFrameAlpha = 1,
		diBgAlpha = 0.5,
		diFrameSize = 1,
		diFrameLock = false,
		diFrameShowRealm = false,
		diExtraWidth = 0,
		diFontSize = 11,
		diTTShow = true,
		diTTShowSpell = false,
		diTTPosition = "ANCHOR_CURSOR",
		-- ISF
		showIsf = true,
		isfLock = false,
		isfInCombat = true,
		isfThreshold = 0,
		isfTop = (UIParent:GetHeight()/2) -40,
		isfLeft = (UIParent:GetWidth()/2),
		isfTextPosition = 0,
		isfFontSize = 10,
		isfAlpha = 1,
		isfWidth = 160,
		isfHeight = 20,
		-- Custom
		showCustom = true,
		soundCustom = true,
		wlrCustomAnchorTop = (UIParent:GetHeight()/2),
		wlrCustomAnchorLeft = (UIParent:GetWidth()/2),
		wlrCustomAnchorAlpha = 1,
		wlrCustomAnchorSize = 1,
		CustomSpells = {
			[1] = 89937,
			[2] = 70840,
		}
    },
}
-- END Default Values

-- Spell Ids
local FelArmor = 28176
local DemonArmor = 687
local SoulLink = 19028
local DrainSoul = 1120
local UnendingBreath = 5697
local SoulFire = 6353

-- Flask
-- Wotlk
local FlaskOfFrostwyrm = 53755
local SpellpowerElixir = 33721
-- Cata
local ElixirOfTheMaster = 79635
local ElixirOfTheMightySpeed = 79632
local ElixirOfImpossibleAccuracy = 79481
local ElixirOfTheCobra = 79477
local FlaskOfTheDraconicMind = 79470
-- Well Fed
local WellFed = 57399 -- there are so many different buffs - using textures

-- Pets
local Pets = {
	["imp"] = 688,
	["voidwalker"] = 697,
	["succubus"] = 712,
	["felhunter"] = 691,
	["felguard"] = 30146,
}

-- Procs
local Decimation1 = 63165
local Decimation2 = 63167
local MoltenCore1 = 47383
local MoltenCore2 = 71162
local MoltenCore3 = 71165
local ShadowTrance = 17941
local EmpoweredImp = 54278
local Isf = 85383

-- Dark Intent
local DarkIntentSpell = 80398
local DarkIntent = 85767
local DarkIntentTarget = ""
local DiScan = {}

-- Custom
local FelSpark = 89937

-- Variables
local wlrDB
local timeLeft = 0
local timeelapsedIsf = 0
local timeelapsedDi = 0
local CustomPlayed = false
local CustomShow = false
local DrainSoulPlayed = false
local DecimationPlayed = false
local ReminderPlayed = false
local _, englishClass = UnitClass("player")
local SelectedSpells = {}

-- Frames
local wlranchor
local wlrframe
local wlrtexture
local diFrame
local diTextureFrame
local diFontFrame
local isfBar
local isfIcon
local wlrcustomanchor
local wlrcustomframe
local wlrcustomtexture


-- Addon start
function WarlockReminder:OnInitialize()
	if(englishClass == "WARLOCK") then
		self.db = LibStub("AceDB-3.0"):New("WarlockReminderDB", defaults, "Default")

		self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileReset", "OnProfileReset")
		self.db.RegisterCallback(self, "OnNewProfile", "OnNewProfile")
		
		wlrDB = self.db.profile
		
		self:CreateFrames()
		self:DiFrameSetup()
		self:IsfSetup()
		
		self:SetUpOptions()
	end
end

function WarlockReminder:OnEnable()
	if(englishClass == "WARLOCK") then
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:RegisterEvent("UNIT_COMBAT")
		self:RegisterEvent("UNIT_AURA")
		self:RegisterEvent("UNIT_PET")
		self:RegisterEvent("COMPANION_UPDATE")
		self:RegisterEvent("UNIT_SPELLCAST_SENT")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		
		-- Anchor Position
		if (not wlrDB.wlrAnchorLeft and not wlrDB.wlrAnchorTop) then
			wlranchor:Show()
			-- When no changes are made, set left and top of the anchor
			wlrDB.wlrAnchorLeft(wlranchor:GetLeft())
			wlrDB.wlrAnchorTop(wlranchor:GetTop())
		else
			self:AnchorSetup()
		end
		
		-- CustomReminder Position
		if (not wlrDB.wlrCustomAnchorLeft and not wlrDB.wlrCustomAnchorTop) then
			wlrcustomanchor:Show()
			-- When no changes are made, set left and top of the anchor
			wlrDB.wlrCustomAnchorLeft(wlrcustomanchor:GetLeft())
			wlrDB.wlrCustomAnchorTop(wlrcustomanchor:GetTop())
		else
			self:CustomSetup()
		end
		
		-- DiFrame Position
		if (not wlrDB.diFrameLeft and not wlrDB.diFrameTop) then
			diFrame:Show()
			-- When no changes are made, set left and top of the frame
			wlrDB.diFrameLeft(diFrame:GetLeft())
			wlrDB.diFrameTop(diFrame:GetTop())
		else
			self:DiFrameSetup()
		end
		
		-- Isf Position
		if (not wlrDB.isfLeft and not wlrDB.isfTop) then
			isfBar:Show()
			-- When no changes are made, set left and top of the frame
			wlrDB.isfLeft(isfBar:GetLeft())
			wlrDB.isfTop(isfBar:GetTop())
		else
			self:IsfSetup()
		end
	end
end
-- END Addon start


local function round(number, decimals)
    return (("%%.%df"):format(decimals)):format(number)
end

-- Check if the unit has a buff, showcount = true returns how often the buff is up
local function UnitHasBuff(unit, buff, showcount)
    for i=1, 40 do
		local name, _, _, count = UnitBuff(unit, i)
        if ( name == GetSpellInfo(buff) ) then
			if(showcount == true) then return count
			else return true end
        end
    end
	return false
end

-- Check if the unit has my buff
local function UnitHasMyBuff(unit, buff)
	for i=1, 40 do
		local buffName, _, _, _, _, _, _, source, _ = UnitBuff(unit, i)
		if (source == "player" and
			source ~=  nil and
			buffName == GetSpellInfo(buff))
		then
			return true
		end
	end
	return false
end

-- Check if the unit has a buff using textures
local function UnitHasBuffByTexture(unit, buff)
	local  _, _, icon = GetSpellInfo(buff)
    for i=1, 40 do
		local _, _, iconTexture = UnitBuff(unit, i)
        if ( iconTexture == icon  ) then
			return true
        end
    end
	return false
end

-- Check if the unit has a buff and return caster
local function UnitHasBuffOfWho(unit, buff)
	for i=1, 40 do
		local buffName, _, _, _, _, _, _, source, _ = UnitBuff(unit, i)
		if (source ~=  nil and
			buffName == GetSpellInfo(buff))
		then
			return source
		end
	end
	return false
end

-- Output Blizzard Message or Chat
function WarlockReminder:Output(input, red, green, blue, duration)
	-- Simulate default values, is there a better way to do this in lua?
	if red == nil then red = 0.0 end
	if green == nil then green = 1.0 end
	if blue == nil then blue = 0.0 end
	if duration == nil then duration = 5.0 end
	
	if wlrDB.outputFrame == "chat" then
		self:Print(input)
	else
		UIErrorsFrame:AddMessage(input, red, green, blue, duration)
	end	
end
-- END Output

-- EVENTS
function WarlockReminder:UNIT_SPELLCAST_SENT(event, unit, spell, _, target)
	if(unit == "player" and spell == GetSpellInfo(DarkIntent)) then
		DarkIntentTarget = target
	end
end

function WarlockReminder:PLAYER_TARGET_CHANGED()
	DrainSoulPlayed = false
	ReminderPlayed = false
	DrainSoulShown = false
	self:DrainSoulReminder()
	self:Reminder()
end

function WarlockReminder:PLAYER_REGEN_DISABLED()
	self:DrainSoulReminder()
	if wlrDB.showIsf == true then isfBar:Show() end
end

function WarlockReminder:PLAYER_REGEN_ENABLED()
	self:ReminderPlayed()
	self:Reminder()
	if wlrDB.isfInCombat == true then isfBar:Hide() end
end

function WarlockReminder:UNIT_COMBAT(unit)
	self:DrainSoulReminder()
end

function WarlockReminder:UNIT_AURA(event, unit)
	if unit == "player" then
		self:ReminderPlayed()
		self:MoltenCoreReminder()
		self:DecimationReminder()
		self:ShadowTranceReminder()
		self:EmpoweredImpReminder()
		self:DrainSoulReminder()
		self:CustomReminder()
		self:Reminder()
	end
end

function WarlockReminder:UNIT_PET(event, unit)
	if unit == "player" then
		self:Reminder()
	end
end

function WarlockReminder:COMPANION_UPDATE(event, companion)
	if companion == "MOUNT" then
		self:Reminder()
	end
end
-- END EVENTS

function WarlockReminder:ReminderPlayed()

	if ( not UnitHasBuff("player", ShadowTrance) 
	) then
		ShadowTrancePlayed = false
	end
	if ( not UnitHasBuff("player", EmpoweredImp) 
	) then
		EmpoweredImpPlayed = false
	end
	if ( not UnitHasBuff("player", Decimation1) and
		not UnitHasBuff("player", Decimation2) 
	) then
		DecimationPlayed = false
	end
	if ( not UnitHasBuff("player", MoltenCore1) and
		not UnitHasBuff("player", MoltenCore2) and
		not UnitHasBuff("player", MoltenCore3)
	) then
		MoltenCorePlayed = false
	end

end

-- Warlock reminder
function WarlockReminder:Reminder()
	local missing = false	
	-- Player not in vehicle and not dead
	if( not UnitInVehicle("player") and
		not UnitIsDead("player")
	) then
		if ( not DrainSoulShown and -- don't show when drain soul is up
			not DrainSoulPlayed and
			not DecimationPlayed and -- don't show when decimation is up
			not MoltenCorePlayed and -- don't show when molten core is up
			not ShadowTrancePlayed and -- don't show when shadow trance is up
			not EmpoweredImpPlayed -- don't show when shadow trance is up
		) then
			-- Bufffood
			if ( wlrDB.showFood and
				UnitLevel("target") < 0 and
				UnitHealth("target") * 100 / UnitHealthMax("target") > 98 and
				not UnitHasBuffByTexture("player", WellFed) ) then
					missing = true
					if wlrDB.outputFrame == "wlrframe" then
						self:WlrFrame(true, WellFed)
					else
						self:Output("* ".. L["Missing Food"] .." *")
					end
			else
				if not missing then self:WlrFrame(false) end
			end
			
			-- Flask & Elixir
			if ( wlrDB.showFlask and
				UnitLevel("target") < 0 and
				UnitHealth("target") * 100 / UnitHealthMax("target") > 98 and
				-- Wotlk
				not UnitHasBuff("player", FlaskOfFrostwyrm) and
				not UnitHasBuff("player", SpellpowerElixir) and
				-- Cata
				not UnitHasBuff("player", ElixirOfTheMaster) and
				not UnitHasBuff("player", ElixirOfTheMightySpeed) and
				not UnitHasBuff("player", ElixirOfImpossibleAccuracy) and
				not UnitHasBuff("player", ElixirOfTheCobra) and
				not UnitHasBuff("player", FlaskOfTheDraconicMind) ) then
					missing = true
					if wlrDB.outputFrame == "wlrframe" then
						self:WlrFrame(true, FlaskOfTheDraconicMind)
					else
						self:Output("* ".. L["Missing Flask"] .." *")
					end
			else
				if not missing then self:WlrFrame(false) end
			end
			
			-- Soul Link
			local slusable, slnomana = IsUsableSpell(SoulLink)
			if( wlrDB.showSL and
				UnitExists("pet") and
				slusable == 1 and -- Spell already learned?
				not IsMounted("player") ) then
					if( not UnitHasBuff("player", SoulLink) ) then
						missing = true
						if wlrDB.outputFrame == "wlrframe" then
							self:WlrFrame(true, SoulLink)
						else
							self:Output("* ".. L["Missing SL"] .." *")
						end
					end
			else
				if not missing then self:WlrFrame(false) end
			end
			
			-- Pet
			if ( wlrDB.showPet and
				not UnitExists("pet") and
				not IsMounted("player") ) then
					missing = true
					if wlrDB.outputFrame == "wlrframe" then
						local pet = ""
						local talents = GetPrimaryTalentTree(false, false, GetActiveTalentGroup(false, false))
						if talents == 1 then pet = Pets[wlrDB.petAffliction] -- Affli
						elseif (talents == 2) then pet = Pets[wlrDB.petDemonology] -- Demo
						else pet = Pets[wlrDB.petDestruction] end -- Destro + unknown
						self:WlrFrame(true, pet)
					else
						self:Output("* ".. L["Missing Pet"] .." *")
					end
			else
				if not missing then	self:WlrFrame(false) end
			end

			-- Armor
			local felarmorusable, felarmornomana = IsUsableSpell(FelArmor)
			local demonarmorusable, demonarmornomana = IsUsableSpell(DemonArmor)
			if ( wlrDB.showArmor and
				-- Spell already learned?
				( felarmorusable == 1 or
				demonarmorusable == 1 ) and
				not UnitHasBuff("player", FelArmor) and -- Fel Armor
				not UnitHasBuff("player", DemonArmor)  -- Demon Armor
			) then
				missing = true
				if wlrDB.outputFrame == "wlrframe" then
					local usearmor = ""
					if felarmorusable == 1 then usearmor = FelArmor
					else usearmor = DemonArmor end
					self:WlrFrame(true, usearmor)
				else 
					self:Output("* ".. L["Missing Armor"] .." *")
				end
			else
				if not missing then self:WlrFrame(false) end
			end
		end
	else
		self:WlrFrame(false)
	end
end
-- END Warlock reminder

-- DrainSoul reminder
function WarlockReminder:DrainSoulReminder()
	if ( not UnitInVehicle("player") and
		not ShadowTrancePlayed and -- don't show when Shadow Trance is up
		UnitCanAttack("player", "target") and
		not UnitIsDead("target") and
		UnitHealth("target") > 0 and
		UnitHealth("target") * 100 / UnitHealthMax("target") <= wlrDB.drainSoulPercent * 100 and
		UnitHealthMax("target") >= tonumber(wlrDB.drainSoulHealth)
		) then
			-- play ds sound
			if ( wlrDB.playDrainSoul and
				not DrainSoulPlayed
				) then
				PlaySoundFile("Interface\\AddOns\\WarlockReminder\\Sounds\\quaddamage.ogg")
			end
			-- show ds message
			if ( wlrDB.showDrainSoulMessage and 
				not DrainSoulPlayed
				) then
				self:Output("* ".. L["Drain Soul!"] .." *", 1.0, 0.0, 0.0)
			end
			-- show ds wlrframe
			if self.drainsoultimer then local temptimer = self.drainsoultimer 
			else temptimer = nil end
			if ( wlrDB.showDrainSoulFrame and
				not self:TimeLeft(temptimer) and
				not DrainSoulShown
			) then
				DrainSoulShown = true
				self:WlrDrainSoulFrame(true)
			end
			DrainSoulPlayed = true
	else 
		DrainSoulPlayed = false
		DrainSoulShown = false
	end
end
-- END DrainSoul reminder

-- Decimation reminder
function WarlockReminder:DecimationReminder()
	if( ( UnitHasBuff("player", Decimation1) or
		UnitHasBuff("player", Decimation2) )
	) then
		-- play decimation sound
		if ( wlrDB.playDecimationSound and
			not DecimationPlayed
		) then
			PlaySoundFile("Interface\\AddOns\\WarlockReminder\\Sounds\\quaddamage.ogg")
		end
		-- show decimation message
		if ( wlrDB.showDecimationMessage and 
			not DecimationPlayed
		) then
			self:Output("* ".. L["Decimation!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show decimation frame
		if  wlrDB.showDecimationFrame then
			self:WlrFrame(true, Decimation2)
		end
		DecimationPlayed = true
	else
		DecimationPlayed = false
	end
end
-- END Decimation reminder

-- moltencore reminder
function WarlockReminder:MoltenCoreReminder()
	if( ( UnitHasBuff("player", MoltenCore1) or
		UnitHasBuff("player", MoltenCore2) or
		UnitHasBuff("player", MoltenCore3) ) and
		not DecimationPlayed  -- don't show when decimation is up
	) then
		-- play moltencore sound
		if ( wlrDB.playMoltenCoreSound and
			not MoltenCorePlayed
		) then
			PlaySoundFile("Sound\\Spells\\Clearcasting_Impact_Chest.wav")
		end
		-- show moltencore message
		if ( wlrDB.showMoltenCoreMessage and 
			not MoltenCorePlayed
		) then
			self:Output("* ".. L["Molten Core!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show moltencore frame
		if  wlrDB.showMoltenCoreFrame then
			-- How often Molten Core is up
			local count = 0
			if UnitHasBuff("player", MoltenCore1) then count = UnitHasBuff("player", MoltenCore1, true)
			elseif UnitHasBuff("player", MoltenCore2) then count = UnitHasBuff("player", MoltenCore2, true)
			else count = UnitHasBuff("player", MoltenCore3, true) end
			
			self:WlrMoltenCoreFrame(true, count)
		end
		MoltenCorePlayed = true
	else
		MoltenCorePlayed = false
	end
end
-- END moltencore reminder

-- ShadowTrance reminder
function WarlockReminder:ShadowTranceReminder()
	if( UnitHasBuff("player", ShadowTrance)
	) then
		-- play shadowtrance sound
		if ( wlrDB.playShadowTranceSound and
			not ShadowTrancePlayed
		) then
			PlaySoundFile("Sound\\Spells\\Clearcasting_Impact_Chest.wav")
		end
		-- show shadowtrance message
		if ( wlrDB.showShadowTranceMessage and 
			not ShadowTrancePlayed
		) then
			self:Output("* ".. L["Shadow Trance!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show shadowtrance frame
		if  wlrDB.showShadowTranceFrame then
			self:WlrFrame(true, ShadowTrance)
		end
		ShadowTrancePlayed = true
	else
		ShadowTrancePlayed = false
	end
end
-- END ShadowTrance reminder

-- EmpoweredImp reminder
function WarlockReminder:EmpoweredImpReminder()
	if(  UnitHasBuff("player", EmpoweredImp)
	) then
		-- play empowered imp sound
		if ( wlrDB.playEmpoweredImpSound and
			not EmpoweredImpPlayed
			) then
			PlaySoundFile("Sound\\Spells\\Clearcasting_Impact_Chest.wav")
		end
		-- show empowered imp message
		if ( wlrDB.showEmpoweredImpMessage and 
			not EmpoweredImpPlayed
			) then
			self:Output("* ".. L["Empowered Imp!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show empowered imp frame
		if  wlrDB.showEmpoweredImpFrame then
			self:WlrFrame(true, EmpoweredImp)
		end
		EmpoweredImpPlayed = true
	else
		EmpoweredImpPlayed = false
	end
end
-- END EmpoweredImp reminder

-- Dark Intent Tracker
function WarlockReminder:DarkIntentTracker()
	local output = 0
	if(	(UnitInRaid(DarkIntentTarget) or UnitInParty(DarkIntentTarget)) and
		UnitHasMyBuff(DarkIntentTarget, DarkIntent) == true
	)then
		local _, diTargetClass = UnitClass(DarkIntentTarget)
		local fontColor
		local diName
		local diRealm
		
		if (not (DarkIntentTarget == "")) then
			local buffName, _, _, _, _, _, expirationTime, _, _ = UnitBuff(DarkIntentTarget, GetSpellInfo(DarkIntent))
			timeLeft = expirationTime
		end
		
		if((-1*(GetTime()-timeLeft)) < 0) then output = ""
		elseif((-1*(GetTime()-timeLeft)) >= 60) then output = format("%.0f",ceil(-1*(GetTime()-timeLeft)/60)).." "..L["minutes"]
		else output = format("%.0f",-1*(GetTime()-timeLeft)).." "..L["seconds"] end
		
		if(wlrDB.diFrameShowRealm == true) then
			diName = DarkIntentTarget
		else
			diName, diRealm = strsplit("-", DarkIntentTarget)
		end
		
		if RAID_CLASS_COLORS[diTargetClass] then
			local fontColor = RAID_CLASS_COLORS[diTargetClass]
			diFontFrame.text:SetTextColor(fontColor["r"], fontColor["g"], fontColor["b"])
		end
		diFontFrame.text:SetText(diName)
		diFontFrame.textTime:SetText(output)
		self:DiFrameSetFontGab()
	else
		timeLeft = 0
		diFontFrame.text:SetTextColor(1, 1, 1)
		diFontFrame.text:SetText(L["No DI"])
		diFontFrame.textTime:SetText("")
	end
end

-- Tooltip
-- Scan party or raid for Di
function WarlockReminder:DarkIntentScan()
	DiScan = {}
	local party = ""
	local count = 1
	if(UnitInRaid("player")) then
		party = "raid"
		count = 40
	elseif(UnitInParty("player")) then
		party = "party"
		count = 4
	end
	
	if(party == "raid" or party == "party") then
		for i = 1, count, 1 do
			local unitCaster = UnitHasBuffOfWho(party..i, DarkIntent)
			if(unitCaster ~= "" and unitCaster ~= nil and unitCaster ~= false) then
				local name, _ = UnitName(party..i)
				local unitCaster, _ = UnitName(unitCaster)
				if(unitCaster ~= name) then
					if(name == nil) then name = " " end
					table.insert(DiScan, unitCaster..";"..name)
				end
			end
		end
	end
end
-- END Dark Intent Tracker

-- Isfreminder
function WarlockReminder:IsfReminder()
	if wlrDB.showIsf then
		local _, _, _, _, _, _, expirationTime = UnitBuff("player", GetSpellInfo(Isf))
		local _, _, _, _, _, _, castTime = GetSpellInfo(SoulFire)

		if not (expirationTime == nil) then
			local isfLeft = (-1*(GetTime()-expirationTime))
			isfBar:SetValue(isfLeft)
			isfBar.text:SetText(round(isfLeft, 1) .. L["seconds"])
			if(tonumber(isfLeft) < ((castTime/1000)+wlrDB.isfThreshold)) then isfBar:SetStatusBarColor(170/255, 10/255, 10/255)
			else isfBar:SetStatusBarColor(10/255, 170/255, 10/255) end
		else
			isfBar:SetValue(0)
			isfBar.text:SetText(L["No Improved Soul Fire"])
		end
	end
end
-- END Isfreminder

-- OnUpdate
function WarlockReminder:OnUpdate(self, elapsed)
	timeelapsedDi = timeelapsedDi + elapsed
	timeelapsedIsf = timeelapsedIsf + elapsed
    if timeelapsedDi >= 1 then
        WarlockReminder:DarkIntentTracker()
        timeelapsedDi = 0
    end
    
    if timeelapsedIsf >= 0.05 then
        WarlockReminder:IsfReminder()
        timeelapsedIsf = 0
    end
end

function WarlockReminder:GetCustomSpells()
	local SpellInfos = {}
	local SpellName = ""
	for index, spell in pairs(wlrDB.CustomSpells) do
		SpellName = GetSpellInfo(spell)
		table.insert(SpellInfos, index, SpellName)
	end
	return SpellInfos
end

function WarlockReminder:GetCurrentSpells(info, key)
	return SelectedSpells[key];
end

function WarlockReminder:SetCurrentSpells(info, key, value)
	if not value then
		value = nil;
	end
	SelectedSpells[key] = value;
end

function WarlockReminder:AddCustomSpell(info, value)
	local found = false
	
	-- Spell already exists?
	for index, spell in pairs(wlrDB.CustomSpells) do
		if spell == value then
			found = true
			break
		end
	end
	
	-- Spell not in list -> add spell
	if found == false then
		table.insert(wlrDB.CustomSpells, value)
	end
end

function WarlockReminder:RemoveCustomSpell(info, value)
	for index, _ in pairs(SelectedSpells) do
		table.remove(wlrDB.CustomSpells, index)
	end
	SelectedSpells = {};
end

-- Custom reminder
function WarlockReminder:CustomReminder()
	if wlrDB.showCustom then
		for _, spell in pairs(wlrDB.CustomSpells) do
			if(UnitHasBuff("player", spell)) then
				self:WlrCustomFrame(true, spell, UnitHasBuff("player", spell, true))
				CustomShow = true
				if (wlrDB.soundCustom and CustomPlayed == false) then
					PlaySoundFile("Sound\\Spells\\Clearcasting_Impact_Chest.wav")
					CustomPlayed = true
				end
				break
			end
		end
		if(CustomShow == false) then 
			CustomPlayed = false
			self:WlrCustomFrame(false)
		end
	end
end
-- END Custom reminder


-- Tooltip
local function OnEnter(self)
	if(wlrDB["diTTShow"] == false) then
		GameTooltip:Hide()
	else
		GameTooltip:SetOwner(self, wlrDB["diTTPosition"])
		if(wlrDB.diTTShowSpell == true) then
			GameTooltip:SetSpellByID(80398)
			GameTooltip:AddLine(" ", 1, 1, 1)
			GameTooltip:AddLine(L["Dark Intent Targets"], 1, 1, 1)
		else
			GameTooltip:SetText(L["Dark Intent"])
		end
		GameTooltip:AddLine(" ", 1, 1, 1)
		
		-- Call the scanner
		WarlockReminder:DarkIntentScan()		
		for _, unitCaster in pairs(DiScan) do
			local diCaster, diTarget = strsplit(";", unitCaster)
			local _, englishClassTarget = UnitClass(diTarget)
			local _, englishClassCaster = UnitClass(diCaster)
			local fontColorTarget = {}
			local fontColorCaster = {}
			if(RAID_CLASS_COLORS[englishClassCaster] == false) then
				fontColorCaster["r"] = 1
				fontColorCaster["g"] = 1
				fontColorCaster["b"] = 1
			else
				fontColorCaster = RAID_CLASS_COLORS[englishClassCaster]
			end
			if(RAID_CLASS_COLORS[englishClassTarget] == false) then
				fontColorTarget["r"] = 1
				fontColorTarget["g"] = 1
				fontColorTarget["b"] = 1
			else
				fontColorTarget = RAID_CLASS_COLORS[englishClassTarget]
			end
			GameTooltip:AddDoubleLine(diTarget, L["by"].." "..diCaster, fontColorTarget["r"], fontColorTarget["g"], fontColorTarget["b"], fontColorCaster["r"], fontColorCaster["g"], fontColorCaster["b"])
		end

		GameTooltip:Show()
	end
end

local function OnLeave(self)
  GameTooltip:Hide()
end

function WarlockReminder:CreateFrames()
	-- Set up anchor
	wlranchor = CreateFrame("Frame", nil, UIParent), {}
	wlranchor:SetWidth(120)
	wlranchor:SetHeight(24)
	wlranchor:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 8, edgeSize = 12, insets = { left = 2, right = 2, top = 2, bottom = 2 } })
	wlranchor:SetBackdropColor(0.1,0.1,0.2,1)
	wlranchor:SetBackdropBorderColor(0.1,0.1,0.1,1)
	wlranchor:SetMovable(1)
	wlranchor:EnableMouse(1)
	wlranchor:SetToplevel(1)
	wlranchor:SetClampedToScreen(1)
	wlranchor:SetPoint("CENTER")
	
	wlranchor.text = wlranchor:CreateFontString(nil,"ARTWORK","GameFontHighlight")
	wlranchor.text:SetText("wlranchor")
	wlranchor.text:SetPoint("LEFT",6,0)

	wlranchor.close = CreateFrame("Button",nil,wlranchor,"UIPanelCloseButton")
	wlranchor.close:SetWidth(24)
	wlranchor.close:SetHeight(24)
	wlranchor.close:SetPoint("RIGHT")
	wlranchor:Hide()
	
	-- Move anchor
	wlranchor:SetScript("OnMouseDown",wlranchor.StartMoving)
	wlranchor:SetScript("OnMouseUp",
		function(self) 
			self:StopMovingOrSizing() 
			-- Set anchor
			wlrDB.wlrAnchorLeft = self:GetLeft()
			wlrDB.wlrAnchorTop = self:GetTop()
			WarlockReminder:FramesToAnchor()
		end
	)
	wlranchor:SetScript("OnEvent",function(self,event,...) self[event](self,event,...) end)
	-- END Move anchor
	-- END Set up anchor

	-- Set up frame
	wlrframe = CreateFrame("Frame", nil, UIParent), {}
	wlrtexture = wlrframe:CreateTexture()
	wlrframe:SetAlpha(1)
	wlrframe:SetPoint("CENTER",wlranchor,0,-42)
	wlrframe:SetWidth(48)
	wlrframe:SetHeight(48)
	wlrtexture:SetAllPoints(wlrframe)
	wlrframe.text = wlrframe:CreateFontString(nil,"OVERLAY","GameFontHighlight")
	wlrframe.text:SetFont("Fonts\\FRIZQT__.TTF", 20, "OUTLINE")
	wlrframe:Hide()
	-- END Set up frame

	-- Set up diFrame
	diFrame = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate"), {}
	diFrame:RegisterForClicks("LeftButtonDown")
	diFrame:SetAttribute("type", "spell")
	diFrame:SetAttribute("unit", "target")
	diFrame:SetAttribute("spell", 80398)

	diFrame:SetAlpha(1)
	diFrame:SetPoint("CENTER", UIParent, 0, 40)
	diFrame:SetWidth(132)
	diFrame:SetHeight(36)

	diFrame:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 8, edgeSize = 12, insets = { left = 2, right = 2, top = 2, bottom = 2 } })
	diFrame:SetBackdropColor(0.1,0.1,0.2,1)
	diFrame:SetBackdropBorderColor(0.1,0.1,0.1,1)
	diFrame:SetMovable(1)
	diFrame:SetToplevel(1)
	diFrame:SetClampedToScreen(1)

	--Set up diTextureFrame
	diTextureFrame = CreateFrame("Frame", nil, diFrame), {}
	diTextureFrame:SetAlpha(0.75)
	diTextureFrame:SetPoint("LEFT", diFrame, 2, 0)
	diTextureFrame:SetWidth(32)
	diTextureFrame:SetHeight(32)
	diTexture = diTextureFrame:CreateTexture()
	diTexture:SetTexture(GetSpellTexture(80398))
	diTexture:SetAllPoints(diTextureFrame)
	diTextureFrame:Show()

	-- Set up diFontFrame
	diFontFrame = CreateFrame("Frame", nil, diFrame), {}
	diFontFrame:SetAlpha(0.75)
	diFontFrame:SetPoint("LEFT", diFrame, diTextureFrame:GetWidth(), 0)
	diFontFrame:SetWidth(diFrame:GetWidth()-32)
	diFontFrame:SetHeight(diFrame:GetHeight() -4)

	diFontFrame.text = diFontFrame:CreateFontString(nil,"ARTWORK","GameFontHighlight")
	diFontFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
	diFontFrame.text:SetText(L["No Dark Intent"])
	diFontFrame.text:SetPoint("LEFT",5,6)

	diFontFrame.textTime = diFontFrame:CreateFontString(nil,"ARTWORK","GameFontHighlight")
	diFontFrame.textTime:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
	diFontFrame.textTime:SetText("")
	diFontFrame.textTime:SetPoint("LEFT",5,-6)
	diFrame:Hide()
		
	diFrame:SetScript("OnUpdate", function(self, elapsed)
		WarlockReminder:OnUpdate(self, elapsed)
	end )
	
	-- Move Dark Intent
	diFrame:SetScript("OnMouseDown", 
		function(self)
			if self:IsMovable() == 1 then
				self:StartMoving()
			end
		end
	)
	diFrame:SetScript("OnMouseUp",
		function(self) 
			if self:IsMovable() == 1 then
				self:StopMovingOrSizing() 
				-- Set diFrame
				wlrDB.diFrameLeft = self:GetLeft()
				wlrDB.diFrameTop = self:GetTop() 
			end
		end
	)
	diFrame:SetScript("OnEvent",function(self,event,...) self[event](self,event,...) end)
	-- END Move Dark Intent
	--END Set up diFrame
	
	-- Set up isfBar
	isfBar = CreateFrame("StatusBar", nil, UIParent), {}
	isfBar:SetMovable(1)
	isfBar:SetToplevel(1)
	isfBar:SetAlpha(0.75)
	isfBar:SetWidth(160)
	isfBar:SetHeight(20)
	isfBar:SetClampedToScreen(1)
	isfBar:SetOrientation("HORIZONTAL")
	isfBar:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
	isfBar:SetStatusBarTexture(media:Fetch("statusbar","Blizzard"))
	isfBar:SetPoint("CENTER", UIParent, 0, 40)
	isfBar:SetStatusBarColor(170/255, 10/255, 10/255)
	isfBar:SetMinMaxValues(0, 15)
	isfBar:Hide()
	
	-- isfText
	isfBar.text = isfBar:CreateFontString(nil,"ARTWORK","GameFontHighlight")
	isfBar.text:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE")
	isfBar.text:SetText(isfBar:GetValue())
	isfBar.text:SetPoint("RIGHT",0,0)
	
	-- isfIcon
	isfIcon = CreateFrame("Frame", nil, isfBar), {}
	isfIcon:SetAlpha(0.75)
	isfIcon:SetPoint("LEFT", isfBar, -1*(isfBar:GetHeight()), 0)
	isfIcon:SetWidth(isfBar:GetHeight())
	isfIcon:SetHeight(isfBar:GetHeight())
	isfIconTexture = isfIcon:CreateTexture()
	isfIconTexture:SetTexture(GetSpellTexture(6353))
	isfIconTexture:SetAllPoints(isfIcon)
	isfIcon:Show()
	
	isfBar:SetScript("OnUpdate", function(self, elapsed)
		WarlockReminder:OnUpdate(self, elapsed)
	end )
	
	-- Move Isf
	isfBar:SetScript("OnMouseDown", 
		function(self)
			if self:IsMovable() == 1 then
				self:StartMoving()
			end
		end
	)
	isfBar:SetScript("OnMouseUp",
		function(self) 
			if self:IsMovable() == 1 then
				self:StopMovingOrSizing() 
				-- Set diFrame
				wlrDB.isfLeft = self:GetLeft()
				wlrDB.isfTop = self:GetTop()  
			end
		end
	)
	isfBar:SetScript("OnEvent",function(self,event,...) self[event](self,event,...) end)
	-- END Move Isf
	--END Set up isfBar
	
	-- Set up customanchor
	wlrcustomanchor = CreateFrame("Frame", nil, UIParent), {}
	wlrcustomanchor:SetWidth(120)
	wlrcustomanchor:SetHeight(24)
	wlrcustomanchor:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 8, edgeSize = 12, insets = { left = 2, right = 2, top = 2, bottom = 2 } })
	wlrcustomanchor:SetBackdropColor(0.1,0.1,0.2,1)
	wlrcustomanchor:SetBackdropBorderColor(0.1,0.1,0.1,1)
	wlrcustomanchor:SetMovable(1)
	wlrcustomanchor:EnableMouse(1)
	wlrcustomanchor:SetToplevel(1)
	wlrcustomanchor:SetClampedToScreen(1)
	wlrcustomanchor:SetPoint("CENTER")
	wlrcustomanchor:Hide()

	wlrcustomanchor.text = wlrcustomanchor:CreateFontString(nil,"ARTWORK","GameFontHighlight")
	wlrcustomanchor.text:SetText("wlrcustomanchor")
	wlrcustomanchor.text:SetPoint("LEFT",6,0)

	wlrcustomanchor.close = CreateFrame("Button",nil,wlrcustomanchor,"UIPanelCloseButton")
	wlrcustomanchor.close:SetWidth(24)
	wlrcustomanchor.close:SetHeight(24)
	wlrcustomanchor.close:SetPoint("RIGHT")
	-- END Set up customanchor
	
	-- Set up customframe
	wlrcustomframe = CreateFrame("Frame", nil, UIParent), {}
	wlrcustomtexture = wlrcustomframe:CreateTexture()
	wlrcustomframe:SetAlpha(1)
	wlrcustomframe:SetPoint("CENTER",wlrcustomanchor,0,-42)
	wlrcustomframe:SetWidth(48)
	wlrcustomframe:SetHeight(48)
	wlrcustomtexture:SetAllPoints(wlrcustomframe)
	wlrcustomframe.text = wlrcustomframe:CreateFontString(nil,"OVERLAY","GameFontHighlight")
	wlrcustomframe.text:SetFont("Fonts\\FRIZQT__.TTF", 20, "OUTLINE")
	wlrcustomframe:Hide()
	-- END Set up customframe
	
	-- Move customanchor
	wlrcustomanchor:SetScript("OnMouseDown",wlrcustomanchor.StartMoving)
	wlrcustomanchor:SetScript("OnMouseUp",
		function(self) 
			self:StopMovingOrSizing() 
			-- Set anchor
			wlrDB.wlrCustomAnchorLeft = self:GetLeft()
			wlrDB.wlrCustomAnchorTop = self:GetTop() 
			WarlockReminder:FramesToAnchor()
		end
	)
	wlrcustomanchor:SetScript("OnEvent",function(self,event,...) self[event](self,event,...) end)
	-- END Move customanchor
end
-- END CreateFrames

-- Resets the anchor to the middle of the screen
function WarlockReminder:ResetAnchor()
	wlranchor:ClearAllPoints()
	wlranchor:SetPoint("CENTER",UIParent,"CENTER", 0, 0)
	wlrDB.wlranchor = wlranchor:GetLeft()
	wlrDB.wlranchor = wlranchor:GetTop() 
	WarlockReminder:FramesToAnchor()
	wlranchor:Show()
end

-- Resets the customanchor to the middle of the screen
function WarlockReminder:ResetCustomAnchor()
	wlrcustomanchor:ClearAllPoints()
	wlrcustomanchor:SetPoint("CENTER",UIParent,"CENTER", 0, 0)
	wlrDB.wlrCustomAnchorLeft = wlrcustomanchor:GetLeft()
	wlrDB.wlrCustomAnchorTop = wlrcustomanchor:GetTop() 
	WarlockReminder:FramesToAnchor()
	wlrcustomanchor:Show()
end

-- Resets the diFrame to the middle of the screen
function WarlockReminder:ResetDiFrame()
	diFrame:ClearAllPoints()
	diFrame:SetPoint("CENTER",UIParent,"CENTER", 0, 40)
	wlrDB.diFrameLeft = diFrame:GetLeft()
	wlrDB.diFrameTop = diFrame:GetTop() 
	diFrame:Show()
end

-- Resets the isfBar to the middle of the screen
function WarlockReminder:ResetIsfBar()
	isfBar:ClearAllPoints()
	isfBar:SetPoint("CENTER",UIParent,"CENTER", 0, -40)
	wlrDB.isfLeft = self:GetLeft()
	wlrDB.isfTop = self:GetTop() 
	isfBar:Show()
end

function WarlockReminder:DiFrameSetFontGab()
	diFontFrame.text:SetPoint("LEFT",5,4+(0.2*diFontFrame.text:GetHeight()))
	diFontFrame.textTime:SetPoint("LEFT",5,-4-(0.2*diFontFrame.textTime:GetHeight()))
end

-- DiFrame Setup
function WarlockReminder:DiFrameSetup()
	diFrame:ClearAllPoints()
	-- Position
	diFrame:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT", wlrDB.diFrameLeft, wlrDB.diFrameTop)
	
	-- Alpha
	diFrame:SetAlpha(wlrDB.diFrameAlpha)
	-- BG Alpha
	diFrame:SetBackdropColor(0.1,0.1,0.2,wlrDB.diBgAlpha)
	diFrame:SetBackdropBorderColor(0.1,0.1,0.1,wlrDB.diBgAlpha)
	
	-- Size
	diFrame:SetWidth((132*wlrDB.diFrameSize)+wlrDB.diExtraWidth)
	diFrame:SetHeight(36*wlrDB.diFrameSize)
	diTextureFrame:SetWidth((36-4)*wlrDB.diFrameSize)
	diTextureFrame:SetHeight((36-4)*wlrDB.diFrameSize)
	diTexture:SetAllPoints(diTextureFrame)
	
	-- Font
	diFontFrame:SetWidth((diFrame:GetWidth()-32)*wlrDB.diFrameSize)
	diFontFrame:SetHeight((diFrame:GetHeight()-4)*wlrDB.diFrameSize)
	diFontFrame:SetPoint("LEFT", diFrame, diTextureFrame:GetWidth(), 0)
	diFontFrame.textTime:SetFont("Fonts\\FRIZQT__.TTF", wlrDB.diFontSize, "OUTLINE")
	diFontFrame.text:SetFont("Fonts\\FRIZQT__.TTF", wlrDB.diFontSize, "OUTLINE")
	self:DiFrameSetFontGab()
	
	-- Movable
	if(wlrDB.diFrameLock == true) then 
		diFrame:SetMovable(0)
	else 
		diFrame:SetMovable(1) 
	end
	
	-- Show/Hide
	if(wlrDB.showDarkIntentFrame) then diFrame:Show()
	else diFrame:Hide() end
	
	-- Tooltip
	if(wlrDB.diTTShow == true) then
		diFrame:SetScript("OnEnter", OnEnter)
		diFrame:SetScript("OnLeave", OnLeave)
	end
end

-- ReminderFrame Setup
function WarlockReminder:AnchorSetup()
	wlranchor:ClearAllPoints()
	
	-- Position
	wlranchor:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT", wlrDB.wlrAnchorLeft, wlrDB.wlrAnchorTop)
	
	-- Alpha
	wlrframe:SetAlpha(wlrDB.wlrAnchorAlpha)

	-- Size
	wlrframe:SetWidth(48*wlrDB.wlrAnchorSize)
	wlrframe:SetHeight(48*wlrDB.wlrAnchorSize)
	
	self:FramesToAnchor()
	
	-- Show/Hide
	if(wlrDB.outputFrame == "wlrframe" or wlranchor:IsShown()) then 
		wlrframe:Show()
	else
		wlrframe:Hide()
		wlranchor:Hide()
	end
end

-- CustomReminder Setup
function WarlockReminder:CustomSetup()
	wlrcustomanchor:ClearAllPoints()
	
	-- Position
	wlrcustomanchor:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT", wlrDB.wlrCustomAnchorLeft, wlrDB.wlrCustomAnchorTop)
	
	-- Alpha
	wlrcustomframe:SetAlpha(wlrDB.wlrCustomAnchorAlpha)
	
	-- Size
	wlrcustomframe:SetWidth(48*wlrDB.wlrCustomAnchorSize)
	wlrcustomframe:SetHeight(48*wlrDB.wlrCustomAnchorSize)
	
	self:FramesToAnchor()
	
	-- Show/Hide
	if(wlrDB.showCustom or wlrcustomanchor:IsShown()) then 
		wlrcustomframe:Show()
	else 
		wlrcustomframe:Hide() 
		wlrcustomanchor:Hide()
	end
end
-- End CustomReminder Setup

-- Isfbar setup
function WarlockReminder:IsfSetup()
	isfBar:ClearAllPoints()
	-- Position
	isfBar:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT", wlrDB.isfLeft, wlrDB.isfTop)
	
	-- Text position
	isfBar.text:ClearAllPoints()
	if (wlrDB.isfTextPosition == 0) then isfBar.text:SetPoint("RIGHT",0,0)
	elseif (wlrDB.isfTextPosition == 1) then isfBar.text:SetPoint("LEFT",0,0)
	else isfBar.text:SetPoint("CENTER",0,0) end
	
	-- Font size
	isfBar.text:SetFont("Fonts\\FRIZQT__.TTF", wlrDB.isfFontSize, "OUTLINE")
	
	-- Alpha
	isfBar:SetAlpha(wlrDB.isfAlpha)
	
	-- Width & Height
	isfBar:SetWidth(wlrDB.isfWidth)
	isfBar:SetHeight(wlrDB.isfHeight)
	isfIcon:SetWidth(isfBar:GetHeight())
	isfIcon:SetHeight(isfBar:GetHeight())
	isfIcon:SetPoint("LEFT", isfBar, -1*(isfBar:GetHeight()), 0)

	-- Movable
	if(wlrDB.isfLock == true) then 
		isfBar:SetMovable(0)
	else 
		isfBar:SetMovable(1) 
	end
	
	-- Show/Hide
	if(wlrDB.showIsf and wlrDB.isfInCombat == false) then
		isfBar:Show()
	else 
		isfBar:Hide()
	end
end

-- Attach all frames to the anchor
function WarlockReminder:FramesToAnchor()
	wlrframe:SetPoint("CENTER",wlranchor,0,-42)
	wlrcustomframe:SetPoint("CENTER",wlrcustomanchor,0,-42) 
end

-- Show the reminder frame
function WarlockReminder:WlrFrame(show, texture)
	if texture then wlrtexture:SetTexture(GetSpellTexture(texture))
	else wlrtexture:SetTexture(GetSpellTexture(28176)) end
	if show then 
		wlrframe.text:SetText("")
		wlrframe:Show()
	else 
		wlrframe:Hide()
	end
end

-- Show the customreminder frame
function WarlockReminder:WlrCustomFrame(show, texture, count)
	if texture then wlrcustomtexture:SetTexture(GetSpellTexture(texture))
	else wlrcustomtexture:SetTexture(GetSpellTexture(89937)) end
	
	if not count or count == 0 then count = "" end
	
	if show then 
		wlrcustomframe.text:SetText(tostring(count))
		wlrcustomframe.text:SetPoint("BOTTOMRIGHT",-3,3)
		wlrcustomframe:Show()
	else 
		wlrcustomframe:Hide()
	end
end

-- Show the drain soul frame
function WarlockReminder:WlrDrainSoulFrame(show)
	if show then 
		wlrtexture:SetTexture(GetSpellTexture(1120))
		wlrframe.text:SetText("")
		wlrframe:Show()
		self:CancelTimer(self.drainsoultimer, true)
		if wlrDB.drainSoulDuration > 0 then
			self.drainsoultimer = self:ScheduleTimer("WlrDrainSoulFrame", wlrDB.drainSoulDuration, false)
		end
	else 
		self:CancelTimer(self.drainsoultimer, true)
		wlrframe:Hide() 
	end
end

-- Show the Molten Core Frame
function WarlockReminder:WlrMoltenCoreFrame(show, count)
	if not count then count = 0 end
	if show then
		wlrtexture:SetTexture(GetSpellTexture(71165))
		wlrframe.text:SetText(tostring(count))
		wlrframe.text:SetPoint("BOTTOMRIGHT",-3,3)
		wlrframe:Show()
	else
		wlrframe:Hide()
	end
end


-- Output frames
local OutputFrames = {
	["chat"] = L["Chat"],
	["blizzard"] = L["Blizzard Message Frame"],
	["wlrframe"] = L["WarlockReminder Frame"],
}
-- Text Positions
local TextPositions = {
	[0] = L["Right"],
	[1] = L["Left"],
	[2] = L["Center"],
}

local PetChoice = {
	["felhunter"] = L["Felhunter"],
	["succubus"] = L["Succubus"],
	["imp"] = L["Imp"],
	["voidwalker"] = L["Voidwalker"],
	["felguard"] = L["Felguard"],
}

local TooltipPositions = {
	["ANCHOR_CURSOR"] = L["Cursor"],
	["ANCHOR_TOPLEFT"] = L["Top left"],
	["ANCHOR_TOPRIGHT"] = L["Top right"],
	["ANCHOR_BOTTOMLEFT"] = L["Bottom left"],
	["ANCHOR_BOTTOMRIGHT"] = L["Bottom right"],
	["ANCHOR_LEFT"] = L["Left"],
	["ANCHOR_RIGHT"] = L["Right"],
}


-- Options
local wlroptions = {
    name = "WarlockReminder",
    handler = WarlockReminder,
    type = "group",
	childGroups = "tree",
    args = {
    	config = {
			type = "execute",
			name = L["Configure"],
			desc = L["Open the options dialog of WarlockReminder"],
			func = function()
				InterfaceOptionsFrame_OpenToCategory(WarlockReminder.optionFrames.profile)
				InterfaceOptionsFrame_OpenToCategory(WarlockReminder.optionFrames.main)
			end,
			guiHidden = true,
		},
		reminder = {
            type = "group",
			order = 0,
            name = L["Reminders"],
            args = {
				armor = {
					type = "toggle",
					order = 10,
					name = L["Armor"],
					desc = L["Show armor reminder"],
					arg = "showArmor",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				sl = {
					type = "toggle",
					order = 20,
					name = L["Soul Link"],
					desc = L["Show Soul Link reminder"],
					arg = "showSL",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				flask = {
					type = "toggle",
					order = 30,
					name = L["Flask and Elixir"],
					desc = L["Show Flask and Elixir reminder"],
					arg = "showFlask",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				food = {
					type = "toggle",
					order = 40,
					name = L["Well Fed"],
					desc = L["Show food reminder"],
					arg = "showFood",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				pet = {
					type = "toggle",
					order = 50,
					name = L["Pet"],
					desc = L["Show pet reminder"],
					arg = "showPet",
					get = "getProfileOption",
					set = "setProfileOption",
					width = "full",
				},
				afflictionpet = {
					type = "select",
					name = L["Affliction"],
					desc = L["Pet for Affliction"],
					order = 60,
					values = PetChoice,
					arg = "petAffliction",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				demonpet = {
					type = "select",
					name = L["Demonology"],
					desc = L["Pet for Demonology"],
					order = 70,
					values = PetChoice,
					arg = "petDemonology",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				destrupet = {
					type = "select",
					name = L["Destruction"],
					desc = L["Pet for Destruction"],
					order = 80,
					values = PetChoice,
					arg = "petDestruction",
					get = "getProfileOption",
					set = "setProfileOption",
				},
			},
		},
		combat = {
            type = "group",
			order = 2,
            name = L["Combat Reminder"],
            args = {
				combatdesc = {
					order = 0,
					name = L["Combat reminder description"],
					type = "description",
				},
				drainsoul = {
					type = "group",
					order = 1,
					name = L["Drain Soul"],
					args = {
						sound = {
							type = "toggle",
							name = L["Play sound reminder"],
							desc = L["Play Drain Soul sound reminder"],
							order = 10,
							arg = "playDrainSoul",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						message = {
							type = "toggle",
							name = L["Message reminder"],
							desc = L["Show Drain Soul message reminder"],
							order = 20,
							arg = "showDrainSoulMessage",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						frame = {
							type = "toggle",
							name = L["Show Drain Soul Frame"],
							desc = L["Drain Soul will be shown in the WarlockReminder Frame"],
							order = 30,
							arg = "showDrainSoulFrame",
							get = "getProfileOption",
							set = "setProfileOption",
							width = "full",
						},
						percent = {
							type = "range",
							name = L["Show reminder at"],
							desc = L["Show Drain Soul reminder at"],
							order = 40,
							min = 0.0,
							max = 1.0,
							step = 0.01,
							isPercent = true,
							arg = "drainSoulPercent",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						health = {
							type = "input",
							name = L["Minimum health"],
							desc = L["Minimum health to show the Drain Soul reminder"],
							order = 50,
							arg = "drainSoulHealth",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						duration = {
							type = "range",
							name = L["Duration of Drain Soul Frame"],
							desc = L["How long the Drain Soul Frame will be shown"],
							order = 60,
							min = 0,
							max = 60,
							step = 1,
							isPercent = false,
							arg = "drainSoulDuration",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				}, -- END Drainsoul
				decimation = {
					type = "group",
					order = 2,
					name = L["Decimation"],
					args = {
						sound = {
							type = "toggle",
							name = L["Play sound reminder"],
							desc = L["Play Decimation sound reminder"],
							order = 1,
							arg = "playDecimationSound",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						message = {
							type = "toggle",
							name = L["Message reminder"],
							desc = L["Show Decimation message reminder"],
							order = 2,
							arg = "showDecimationMessage",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						frame = {
							type = "toggle",
							name = L["Show Decimation Frame"],
							desc = L["Show Decimation Frame"],
							order = 3,
							arg = "showDecimationFrame",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				}, -- END Decimation
				moltencore = {
					type = "group",
					order = 3,
					name = L["Molten Core"],
					args = {
						sound = {
							type = "toggle",
							name = L["Play sound reminder"],
							desc = L["Play Molten Core sound reminder"],
							order = 1,
							arg = "playMoltenCoreSound",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						message = {
							type = "toggle",
							name = L["Message reminder"],
							desc = L["Show Molten Core message reminder"],
							order = 2,
							arg = "showMoltenCoreMessage",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						frame = {
							type = "toggle",
							name = L["Show Molten Core Frame"],
							desc = L["Show Molten Core Frame"],
							order = 3,
							arg = "showMoltenCoreFrame",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				}, -- END Molten Core
				shadowtrance = {
					type = "group",
					order = 4,
					name = L["Shadow Trance"],
					args = {
						sound = {
							type = "toggle",
							name = L["Play sound reminder"],
							desc = L["Play Shadow Trance sound reminder"],
							order = 1,
							arg = "playShadowTranceSound",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						message = {
							type = "toggle",
							name = L["Message reminder"],
							desc = L["Show Shadow Trance message reminder"],
							order = 2,
							arg = "showShadowTranceMessage",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						frame = {
							type = "toggle",
							name = L["Show Shadow Trance Frame"],
							desc = L["Show Shadow Trance Frame"],
							order = 3,
							arg = "showShadowTranceFrame",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				}, -- END Shadow Trance
				empoweredimp = {
					type = "group",
					order = 5,
					name = L["Empowered Imp"],
					args = {
						sound = {
							type = "toggle",
							name = L["Play sound reminder"],
							desc = L["Play Empowered Imp sound reminder"],
							order = 1,
							arg = "playEmpoweredImpSound",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						message = {
							type = "toggle",
							name = L["Message reminder"],
							desc = L["Show Empowered Imp message reminder"],
							order = 2,
							arg = "showEmpoweredImpMessage",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						frame = {
							type = "toggle",
							name = L["Show Empowered Imp Frame"],
							desc = L["Show Empowered Imp Frame"],
							order = 3,
							arg = "showEmpoweredImpFrame",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				},-- END Empowered Imp
			},
		},
		output = {
            type = "group",
			order = 2,
            name = L["Output Frame"],
            args = {
				frame = {
					type = "select",
					name = L["Output Frame"],
					desc = L["Reminders will be shown in the selected frame"],
					order = 1,
					values = OutputFrames,
					arg = "outputFrame",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				anchorheader = {
					type = "header",
					order = 2,
					name = L["WarlockReminder Frame"],
				},
				show = {
					type = "execute",
					name = L["Show"],
					desc = L["Anchor Show"],
					order = 3,
					func = function() wlranchor:Show()
								WarlockReminder:WlrFrame(true, FelArmor)
							end,
				},
				reset = {
					type = "execute",
					name = L["Reset position"],
					desc = L["Anchor Reset"],
					order = 4,
					func = "ResetAnchor",
				},
				alpha = {
					type = "range",
					name = L["Frame alpha"],
					desc = L["Frame alpha"],
					order = 5,
					min = 0.0,
					softMin = 0.25,
					max = 1.0,
					step = 0.05,
					isPercent = true,
					arg = "wlrAnchorAlpha",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				size = {
					type = "range",
					name = L["Frame size"],
					desc = L["Frame size"],
					order = 6,
					min = 0.01,
					softMin = 0.25,
					max = 3,
					softMax = 2,
					step = 0.01,
					isPercent = true,
					arg = "wlrAnchorSize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
			},
		}, -- END output
		darkintent = {
            type = "group",
			order = 3,
            name = L["Dark Intent tracker"],
            args = {
				show = {
					type = "toggle",
					order = 10,
					name = L["Show tracker"],
					desc = L["Show Dark Intent tracker"],
					arg = "showDarkIntentFrame",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				reset = {
					type = "execute",
					name = L["Reset position"],
					desc = L["Dark Intent tracker reset"],
					order = 20,
					func = "ResetDiFrame",
				},
				lock = {
					type = "toggle",
					name = L["Lock frame"],
					desc = L["Lock frame"],
					order = 30,
					arg = "diFrameLock",
					get = "getProfileOption",
					set = "SetDarkIntentLock",
				},
				realm = {
					type = "toggle",
					name = L["Show realm"],
					desc = L["Show realm"],
					order = 40,
					arg = "diFrameShowRealm",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				alpha = {
					type = "range",
					name = L["Frame alpha"],
					desc = L["Frame alpha"],
					order = 50,
					min = 0.0,
					softMin = 0.25,
					max = 1.0,
					step = 0.05,
					isPercent = true,
					arg = "diFrameAlpha",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				bgalpha = {
					type = "range",
					name = L["Background alpha"],
					desc = L["Background alpha"],
					order = 60,
					min = 0.0,
					max = 1.0,
					step = 0.05,
					isPercent = true,
					arg = "diBgAlpha",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				size = {
					type = "range",
					name = L["Frame size"],
					desc = L["Frame size"],
					order = 70,
					min = 0.01,
					softMin = 0.5,
					max = 5,
					softMax = 2,
					step = 0.01,
					isPercent = true,
					arg = "diFrameSize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				extrawidth = {
					type = "range",
					name = L["Extra width"],
					desc = L["Extra width"],
					order = 80,
					min = 0.00,
					max = 1000,
					softMax = 100,
					step = 1,
					arg = "diExtraWidth",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				fontsize = {
					type = "range",
					name = L["Font size"],
					desc = L["Font size"],
					order = 90,
					min = 0.00,
					softMin = 7,
					max = 32,
					step = 1,
					arg = "diFontSize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				ttheader = {
					type = "header",
					name = L["Tooltip"],
					order = 100,
					width = "full",
				},
				ttshow = {
					type = "toggle",
					name = L["Show tooltip"],
					desc = L["Show tooltip"],
					order = 110,
					arg = "diTTShow",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				ttspell = {
					type = "toggle",
					name = L["Show spell tooltip"],
					desc = L["Show spell tooltip"],
					order = 120,
					arg = "diTTShowSpell",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				ttanchor = {
					type = "select",
					name = L["Tooltip Position"],
					desc = L["Tooltip Position"],
					order = 130,
					values = TooltipPositions,
					arg = "diTTPosition",
					get = "getProfileOption",
					set = "setProfileOption",
				},
			},
		}, -- END Dark Intent
		isftimer = {
			type = "group",
			order = 4,
			name = L["Improved Soul Fire timer"],
			args = {
				toggle = {
					type = "toggle",
					order = 10,
					name = L["Show Improved Soul Fire timer"],
					desc = L["Show Improved Soul Fire timer"],
					arg = "showIsf",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				show = {
					type = "execute",
					name = L["Show"],
					desc = L["Show Improved Soul Fire timer"],
					order = 20,
					func = function() isfBar:Show() end,
				},
				lock = {
					type = "toggle",
					name = L["Lock frame"],
					desc = L["Lock frame"],
					order = 30,
					arg = "isfLock",
					get = "getProfileOption",
					set = "SetIsfLock",
				},
				reset = {
					type = "execute",
					name = L["Reset position"],
					desc = L["Improved Soul Fire reset"],
					order = 40,
					func = "ResetIsfBar",
				},
				combat = {
					type = "toggle",
					name = L["Show only in combat"],
					desc = L["Show only in combat"],
					order = 50,
					arg = "isfInCombat",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				textorientation = {
					type = "select",
					name = L["Text orientation"],
					desc = L["Text orientation"],
					order = 60,
					values = TextPositions,
					arg = "isfTextPosition",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				fontsize = {
					type = "range",
					name = L["Font size"],
					desc = L["Font size"],
					order = 70,
					min = 0.00,
					softMin = 7,
					max = 32,
					step = 1,
					arg = "isfFontSize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				alpha = {
					type = "range",
					name = L["Frame alpha"],
					desc = L["Frame alpha"],
					order = 80,
					min = 0.0,
					softMin = 0.25,
					max = 1.0,
					step = 0.05,
					isPercent = true,
					arg = "isfAlpha",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				width = {
					type = "range",
					name = L["Width"],
					desc = L["Width"],
					order = 90,
					min = 0.00,
					max = 1000,
					softMax = 400,
					step = 1,
					arg = "isfWidth",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				height = {
					type = "range",
					name = L["Height"],
					desc = L["Height"],
					order = 100,
					min = 0.00,
					max = 100,
					softMax = 50,
					step = 1,
					arg = "isfHeight",
					get = "getProfileOption",
					set = "setProfileOption",
				},
			},
		}, -- END Isf
		-- CustomReminder
		customreminder = {
			type = "group",
			order = 5,
			name = L["Custom Reminder"],
			args = {
				customanchorheader = {
					type = "header",
					order = 10,
					name = L["Custom Reminder"],
				},
				show = {
					type = "toggle",
					order = 20,
					name = L["Show Custom Reminder"],
					desc = L["Show Custom Reminder"],
					arg = "showCustom",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				showanchor = {
					type = "execute",
					name = L["Show"],
					desc = L["Anchor Show"],
					order = 30,
					func = function(self) wlrcustomanchor:Show()
							WarlockReminder:WlrCustomFrame(true, FelSpark)
						end,
				},
				sound = {
					type = "toggle",
					order = 40,
					name = L["Play sound reminder"],
					desc = L["Play sound reminder"],
					arg = "soundCustom",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				
				reset = {
					type = "execute",
					name = L["Reset position"],
					desc = L["Anchor Reset"],
					order = 50,
					func = "ResetCustomAnchor",
				},
				alpha = {
					type = "range",
					name = L["Frame alpha"],
					desc = L["Frame alpha"],
					order = 60,
					min = 0.0,
					softMin = 0.25,
					max = 1.0,
					step = 0.05,
					isPercent = true,
					arg = "wlrCustomAnchorAlpha",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				size = {
					type = "range",
					name = L["Frame size"],
					desc = L["Frame size"],
					order = 70,
					min = 0.01,
					softMin = 0.25,
					max = 3,
					softMax = 2,
					step = 0.01,
					isPercent = true,
					arg = "wlrCustomAnchorSize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				remindersetup = {
					type = "group",
					order = 1,
					name = L["Spell setup"],
					args = {
						addspell = {
							name = L["Add a Spell"],
							desc = L["Add a new spell to the list by entering the Spell-Id here."],
							type = "input",
							order = 10,
							get = false,
							set = "AddCustomSpell",
							width = "double",
						},
						selectspell = {
							name = "",
							desc = "",
							type = "multiselect",
							order = 20,
							get = "GetCurrentSpells",
							set = "SetCurrentSpells",
							values = "GetCustomSpells",
							width = "double",
						},
						removespell = {
							name = L["Remove Spell"],
							desc = L["Remove the selected spell from the list."],
							type = "execute",
							func = "RemoveCustomSpell",
							order = 30,
						},
						customdesc = {
							order = 100,
							name = L["Use spell-ids from wowhead.com's Urls, e.g. Devious Minds: http://www.wowhead.com/spell=70840 => Spell-Id: 70840"],
							type = "description",
						},
					},
				},
			},
		}, -- END CustomReminder
    },
}
-- END Options

function WarlockReminder:OnProfileChanged(event, database, newProfileKey)
	wlrDB = self.db.profile
	self:AnchorSetup()
	self:CustomSetup()
	self:DiFrameSetup()
	self:IsfSetup()
end

function WarlockReminder:OnProfileReset(event, database, newProfileKey)
	wlrDB = self.db.profile
	self:ResetAnchor()
	self:ResetCustomAnchor()
	self:ResetDiFrame()
end

function WarlockReminder:OnNewProfile(event, database, newProfileKey)
	wlrDB = self.db.profile
	self:ResetAnchor()
	self:ResetCustomAnchor()
	self:ResetDiFrame()
end

-- Setup options
function WarlockReminder:SetUpOptions()
	local wlrprofile = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)
	-- Add dual-spec support
	local LibDualSpec = LibStub("LibDualSpec-1.0", true)
	if LibDualSpec then
		LibDualSpec:EnhanceDatabase(self.db, "WarlockReminder")
		LibDualSpec:EnhanceOptions(wlrprofile, self.db)
	end
	
	local config = LibStub("AceConfig-3.0")
	config:RegisterOptionsTable("WarlockReminder", wlroptions, {"wlr", "warlockreminder"})
	
	local registry = LibStub("AceConfigRegistry-3.0")
	-- Add option windows
    registry:RegisterOptionsTable("WarlockReminder Main", wlroptions)
	registry:RegisterOptionsTable("WarlockReminder Profiles", wlrprofile)
	
	local dialog = LibStub("AceConfigDialog-3.0");
	self.optionFrames = {
		main = dialog:AddToBlizOptions("WarlockReminder Main", "WarlockReminder"),
		profile = dialog:AddToBlizOptions("WarlockReminder Profiles", L["Profiles"], "WarlockReminder"),
	}
end
-- END Setup options


function WarlockReminder:getProfileOption(info)
	return wlrDB[info.arg]
end

function WarlockReminder:setProfileOption(info, value)
	wlrDB[info.arg] = value
	if info.uiType == "cmd" then
		WarlockReminder:Print(L["|cffffff7f%s|r is now set to |cffffff7f[%s]|r"]:format(info.option.name, tostring(value)))
	end
	
	self:DiFrameSetup()
	self:CustomSetup()
	self:AnchorSetup()
	self:IsfSetup()
end

function WarlockReminder:SetDarkIntentLock(info, newValue)
    wlrDB.diFrameLock = newValue
	if(newValue == true) then 
		diFrame:SetMovable(0)
	else 
		diFrame:SetMovable(1) 
	end
	if info.uiType == "cmd" then
		WarlockReminder:Print(L["|cffffff7f%s|r is now set to |cffffff7f[%s]|r"]:format(info.option.name, tostring(value)))
	end
end

function WarlockReminder:SetIsfLock(info, newValue)
    wlrDB.isfLock = newValue
	if(newValue == true) then 
		isfBar:SetMovable(0)
	else 
		isfBar:SetMovable(1) 
	end
	if info.uiType == "cmd" then
		WarlockReminder:Print(L["|cffffff7f%s|r is now set to |cffffff7f[%s]|r"]:format(info.option.name, tostring(value)))
	end
end

